<?php

namespace Drupal\openai_client\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\openai_client\OpenAIClientWrapper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure OpenAI Client settings for this site.
 *
 * @see chat_workflow https://platform.openai.com/docs/guides/chat
 */
class SettingsForm extends ConfigFormBase {

  /**
   * @var \Drupal\openai_client\OpenAIClientWrapper
   */
  private OpenAIClientWrapper $api;

  /**
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  private DateFormatter $dateFormatter;

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   */
  public function __construct(OpenAIClientWrapper $api, ConfigFactoryInterface $config_factory, DateFormatter $dateFormatter) {
    parent::__construct($config_factory);

    $this->api = $api;
    $this->dateFormatter = $dateFormatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('openai_client.wrapper'),
      $container->get('config.factory'),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'openai_client_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['openai_client.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    // Help links.
    $links = [];
    $links['openai-documentation'] = [
      '#type' => 'link',
      '#title' => self::t('OpenAI API documentation'),
      '#url' => Url::fromUri('https://platform.openai.com/docs/introduction/overview'),
      '#attributes' => [
        'target' => '_blank',
        'rel' => 'noopener noreferrer',
      ],
    ];

    $links['openai-api-reference'] = [
      '#type' => 'link',
      '#title' => self::t('OpenAI API reference'),
      '#url' => Url::fromUri('https://platform.openai.com/docs/api-reference'),
      '#attributes' => [
        'target' => '_blank',
        'rel' => 'noopener noreferrer',
      ],
    ];

    $links['openai-php-github'] = [
      '#type' => 'link',
      '#title' => self::t('OpenAI PHP in Github'),
      '#url' => Url::fromUri('https://github.com/openai-php/client'),
      '#attributes' => [
        'target' => '_blank',
        'rel' => 'noopener noreferrer',
      ],
    ];

    $form['doc-links'] = [
      '#theme' => 'item_list',
      '#items' => $links,
      '#title' => t('Documentation links'),
    ];

    // Settings form.
    $apiToken = $this->config('openai_client.settings')->get('api-token');
    $form['api-token'] = [
      '#type' => 'password',
      '#title' => $this->t('OpenAI API Token'),
      '#default_value' => $apiToken,
      '#description' => (!empty($apiToken) ? $this->t('The token is set.') : ''),
    ];

    if ($this->api->hasToken()) {
      $form['api-chat-model'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Chat model'),
        '#default_value' => $this->config('openai_client.settings')->get('api-chat-model'),
      ];

      try {
        $models = $this->api->getModels();
      } catch (\Exception $e) {
        $models = [];
      }
      if (count($models) > 0) {
        $modelGroups = [];
        foreach ($models as $i => $model) {
          if (!isset($modelGroups[$models[$i]['owned_by']])) {
            $modelGroups[$models[$i]['owned_by']] = [];
          }
          $modelGroups[$models[$i]['owned_by']][] = $model;
        }
        foreach ($modelGroups as $owner => &$modelGroup) {
          usort($modelGroup, function($a, $b) {
            if ($a['created'] == $b['created']) {
              return 0;
            }
            return ($a['created'] < $b['created']) ? 1 : -1;
          });
          $form['api-models-'.$owner] = array(
            '#type' => 'table',
            '#caption' => $this->t('@owner models', ['@owner' => $owner]),
            '#header' => [
              '#',
              $this->t('ID'),
              $this->t('Object'),
              $this->t('Created'),
              $this->t('Owner'),
              $this->t('root'),
            ],
          );
          foreach ($modelGroup as $i => $model) {
            $form['api-models-'.$owner][$i]['index'] = ['#type' => 'item', '#markup' => $i];
            $form['api-models-'.$owner][$i]['id'] = ['#type' => 'item', '#markup' => $model['id']];
            $form['api-models-'.$owner][$i]['object'] = ['#type' => 'item', '#markup' => $model['object']];
            $date = date($model['created']);
            $form['api-models-'.$owner][$i]['created'] = ['#type' => 'item', '#markup' => $this->dateFormatter->format($date, 'Y-m-d')];
            $form['api-models-'.$owner][$i]['owned_by'] = ['#type' => 'item', '#markup' => $model['owned_by']];
            $form['api-models-'.$owner][$i]['root'] = ['#type' => 'item', '#markup' => $model['root']];
          }
        }
      }
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $token = $form_state->getValue('api-token');
    if (empty($token)) {
      $token = $this->config('openai_client.settings')->get('api-token');
    }
    if (!empty($token)) {
      try {
        $this->api->getClient($token)->models()->list();
      } catch (\Exception $e) {
        $form_state->setErrorByName('api-token', $e->getMessage());
      }
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('openai_client.settings');
    if (!empty($form_state->getValue('api-token'))) {
      $config->set('api-token', trim($form_state->getValue('api-token')));
    }
    $model = $form_state->getValue('api-chat-model');
    if (empty($model)) {
      $model = '';
    }
    $config->set('api-chat-model', trim($model));
    $config->save();
    parent::submitForm($form, $form_state);
  }

}
